<?php declare(strict_types=1);

namespace Picea;

class Builder
{
    const TEMPLATE_CLASSNAME_PREFIX = "PiceaTemplate_";

    protected string $templatePath = "";

    public function __construct(string $templatePath = "./Builder/ClassTemplate.php")
    {
        $this->templatePath = $templatePath;
    }

    public function build(Compiler\Context &$context, string $compiledSource) : array
    {
        $className = static::generateClassName($compiledSource);

        $replace = [
            '%NAMESPACE%' => $context->namespace,
            '%USE%' => $context->renderUses(),
            '%CLASSNAME%' => $className,
            '%EXTENDS%' => $context->extendFrom ? static::generateClassUID($context->extendFrom) : '',
            '%CONTENT%' => $compiledSource,
        ];

        $classContent = str_replace(array_keys($replace), array_values($replace), file_get_contents($this->templatePath));

        return [
            $context->namespace, $className, $classContent,
        ];
    }

    public static function generateClassName(string $filepath) : string
    {
        return static::TEMPLATE_CLASSNAME_PREFIX . static::generateClassUID($filepath);
    }

    public static function generateClassUID(string $filePath) : string
    {
        return md5($filePath);
    }
}
